package bitcreek;

import java.io.Serializable;
import java.net.InetSocketAddress;

/**
 * Questa classe definisce in modo univoco l'oggetto <code>Peer</code> come
 * una coppia di informazioni indirizzo:porta.<br>
 * Gli attributi di questa classe, che sono dichiarati <code>final</code>
 * e hanno visibilita' pubblica, sono solo due:
 * l'oggetto InetSocketAddress idPeer, che rappresenta l'
 * <code>indirizzo:porta</code> del client e l'intero <code>hash</code>,
 * che mantiene l'hashcode univoco dell'oggetto idPeer.<br>
 * La classe effettua l'override dei metodi <code>hashCode</code> e
 * <code>equals</code>, al fine di poter comparare le varie istanze
 * dell'oggetto.<br>
 * Non ha bisogno di essere synchronized perche' l'oggeto e' immutabile.
 * @author  Giulio Tranchida (gtranchi) no. matricola 241732<br>
 * 	     	Progetto LPR anno 2008/2009. */
public class Peer implements Serializable{
	private static final long serialVersionUID = -1908454000720909484L;
	/** Quest'oggeto ha visibilita' publica per ottimizzare l'efficenza del
	 * codice, questo non crea problemi perche' e' comunque definito come 
	 * <code>final</code> */
	public final InetSocketAddress idPeer;
	/** Quest'oggeto ha visibilita' publica per ottimizzare l'efficenza del
	 * codice, questo non crea problemi perche' e' comunque definito come 
	 * <code>final</code> */	
	public final int hash;
	
	/** Costruttore. Crea un nuovo Peer.
	 * @param idPeer
	 * @throws NullPointerException */
	public Peer(InetSocketAddress idPeer) throws NullPointerException{
		if (idPeer==null) throw new NullPointerException(
				"L'InetSocketAddress non puo' essere nullo!");
		this.idPeer = idPeer;
		this.hash = idPeer.hashCode();
	}
	
	@Override
	public int hashCode(){return hash;}

	@Override
	public boolean equals(Object obj){return idPeer.equals(obj);}

}