package bitcreek.server;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLSocket;

import bitcreek.Message;

/**
 * @author  Giulio Tranchida (gtranchi) no. matricola 241732<br>
 * 	     	Progetto LPR anno 2008/2009. */
public class TrackerTCP implements Runnable {
	private boolean shutdown = false;	
	/** Lista degli swarm associata al tracker */
	protected final SwarmList swarmList;
	private SSLSocket clientSslSk;
	private SSLServerSocket serverSslSK = null;
	private ServerRMIImpl server = null;
	
	/** Costruttore della classe.
	 * Inizializza setta lo swarm su cui lavorera' il thread
	 * @param server Istanza della classe cle implementa il server, usata
	 *               solo al fine di rimuovere il descrittore di file quando 
	 *               il suo swarm e' vuoto.
	 * @param swarmList Lo swarm ove questo thread lavorera'   */
	public TrackerTCP(ServerRMIImpl server, SwarmList swarmList){
		this.swarmList = swarmList;
		serverSslSK = swarmList.getSslServerSK();
		this.server = server;
	}
	
	/** Arresta il tracker TCP */
	public void shutdown(){
        shutdown = true;
        try {
       		clientSslSk.close();
        	serverSslSK.close();
        }
        catch (Exception ignore){}
    }
	
	public void run(){
		ServerSwarm swarm = null;
		try {
			while(!shutdown){
				swarm = null;
				clientSslSk = (SSLSocket) serverSslSK.accept();
				ObjectInputStream netIn =
					new ObjectInputStream(clientSslSk.getInputStream());
				ObjectOutputStream netOut =
					new ObjectOutputStream(clientSslSk.getOutputStream());
				
				try { 
					Message msg = (Message) netIn.readObject();
					switch(msg.code){
					case Message.SWARM_REQUEST:
						Server.log.out("[Tracker TCP] Peer "+msg.idPeer
							+" ha richiesto lo swarm del file '"
							+msg.filename+"'.");
						synchronized(this){
							swarm = swarmList.getSwarm(msg.filename);
							swarm.checkOldPeer();
							if (swarm.size()==0){
								/* Lo swarm e' vuoto e lo rimuovo dalla
								 * lista degli swarm gestita da questo
								 * tracker */
								swarmList.removeSwarm(msg.filename);
								swarm = null;
								/* Rimuovo il descrittore dalla lista dei
								 * descrittori dei file pubblicati sul
								 * Server */
								server.removeDescriptor(msg.filename);
							}
						}
						if (swarm!=null){
							netOut.writeObject(
								Message.swarmFound(swarm.getPeerList()));
							/* Il peer che ha richiesto lo swarm viene
							 * aggiunto allo swarm stesso, se non gia'
							 * presente. */
							if (!swarm.existPeer(msg.idPeer))
								swarm.newPeer(msg.idPeer);
						}
						else{
							netOut.writeObject(Message.swarmNotFound());
							Server.log.err("[Tracker TCP] ERRORE Swarm '"
								+msg.filename+"' richiesto dal Peer "
								+msg.idPeer+" inesistente!");						
						}
						clientSslSk.close();
						break;
					default:
						Server.log.err("[Tracker TCP] ERRORE Codice" +
								" messaggio non riconosciuto!");
						break;
					}
				}
				catch (InterruptedException e){
					clientSslSk.close();
					Server.log.err("[Tracker TCP] ERRORE Interruzione non"+
						" prevista.\nLo Swarm potrebbe essere in uno stato"+
						" inconsistente: ");
					Server.log.err(e);
				}
				catch (ClassNotFoundException e){
					clientSslSk.close();
					Server.log.err("[Tracker TCP] ERRORE Messaggio errato"+
						" ricevuto dal Peer.\n");
					Server.log.err(e);
				}
				catch (NullPointerException e){
					Server.log.err("[Tracker TCP] ERRORE passato un"+
						" parametro nullo.");					
					Server.log.err(e);
					clientSslSk.close();
				}
			}
		}
		catch (IOException e){
			if (!shutdown){
				Server.log.err(e.getMessage());
				Server.log.err(e);
			}
		}
		Server.log.err("[TRACKER TCP] Arrestato.");
	}
}
