package bitcreek.server;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.DatagramPacket;

import bitcreek.Message;

/**
 * @author  Giulio Tranchida (gtranchi) no. matricola 241732<br>
 * 	     	Progetto LPR anno 2008/2009. */
public class TrackerUDP implements Runnable {
	/** */
	public static final int MAX_UDP_PAYLOAD = 1024;
	/** */
	public static int BUFFER_SIZE = 1024;
	private boolean shutdown;
	/** Pacchetto Datagram che ricevo */
	private DatagramPacket dp_receive = null;
	/** Lista degli swarm associata al tracker */
	private SwarmList swarmList = null;

	/** Costruttore
	 * @param swarmList */
	public TrackerUDP(SwarmList swarmList){
		this.swarmList = swarmList;
		shutdown = false;
	}
    
	/** Arresta il tracker UDP */
	public void shutdown(){
        shutdown = true;
        swarmList.getSkUDP().close();
    }
	
	/** Restituisce l'oggetto contenuto nel pacchetto passato per argomento,
	 *  deserializzandolo.
	 * @param dp DatagramPacket
	 * @return Restituisce l'oggetto contenuto nel pacchetto passato per
	 *         argomento, deserializzandolo.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public static Object dp2obj(DatagramPacket dp)
	throws IOException, ClassNotFoundException{		
		Object obj = null;
		ByteArrayInputStream bin = 
			new ByteArrayInputStream(dp.getData(), dp.getOffset(), dp.getLength());
		ObjectInputStream object_input_stream =
			new ObjectInputStream(bin);
		obj = object_input_stream.readObject();
		return obj;	
	}
		
	public void run(){
		while(!shutdown){
			try {			
				dp_receive = new DatagramPacket(
						new byte[BUFFER_SIZE], BUFFER_SIZE);
				swarmList.getSkUDP().receive(dp_receive);
				Message msg = (Message) dp2obj(dp_receive);
				switch(msg.code){
				case Message.KEEPALIVE:
					Server.log.out("[KeepAlive] Dal Peer "+msg.idPeer+
						" per descrittore '"+msg.filename+ "'.");
					swarmList.keepAlive(msg.filename, msg.idPeer);
					break;	
				default:
					Server.log.err("[KeepAlive] ERRORE Codice " 
						+"messaggio non riconosciuto!");
					break;
				}
			}
			catch (Exception e){
				if (!shutdown){
					Server.log.err("[KeepAlive] ERRORE "+e.getMessage());
					Server.log.err(e);
				}
			}
		}
		Server.log.out("[KeepAlive] Arrestato.");
	}
}
